/*
 * Copyright 2018 ScaleDynamics SAS. All rights reserved.
 * Licensed under the MIT license.
 */

'use strict';

const runtimeCapabilities = Object.freeze({});

function getRuntimeCapabilities() {
    if (global.warp && global.warp.runtime && global.warp.runtime.installWarpAPI) {
        global.warp.runtime.installWarpAPI(require('./package.json').version, module.exports);
        if (getRuntimeCapabilities === module.exports.getRuntimeCapabilities) {
            throw new Error(
                `global.warp.runtime.installWarpAPI failed to install functions ${module.exports}`);
        }
        return module.exports.getRuntimeCapabilities();
    } else {
        return runtimeCapabilities;
    }
}

function getInstallFunction(name) {
    return (...args) => {
        if (global.warp && global.warp.runtime && global.warp.runtime.installWarpAPI) {
            const installFunction = module.exports[name];
            global.warp.runtime.installWarpAPI(require('./package.json').version, module.exports);
            if (installFunction === module.exports[name]) {
                throw new Error(
                    `global.warp.runtime.installWarpAPI failed to install functions ${module.exports}`);
            }
            return module.exports[name](...args);
        } else {
            throw new Error(
                'No warp runtime, see https://redirect.scaledynamics.io/error/W001 for details');
        }
    };
}

module.exports = {
    call: getInstallFunction('call'),
    callAsPromise: getInstallFunction('callAsPromise'),
    getCurrentNumWarpThreads: getInstallFunction('getCurrentNumWarpThreads'),
    getRuntimeCapabilities,
    markTransferable: getInstallFunction('markTransferable'),
};
